REBOL [
	Title: "REBOL Preprocessor"
	Version: 2.7.8
	Rights: "Copyright REBOL Technologies 2011. All rights reserved."
	Home: http://www.rebol.com
	Date: 1-Jan-2011

	; You are free to use, modify, and distribute this file as long as the
	; above header, copyright, and this entire comment remains intact.
	; This software is provided "as is" without warranties of any kind.
	; In no event shall REBOL Technologies or source contributors be liable
	; for any damages of any kind, even if advised of the possibility of such
	; damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes. See http://www.rebol.com/support.html for details.
	; You are free to use, modify, and distribute this software with any
	; REBOL Technologies products as long as the above header, copyright,
	; and this comment remain intact. This software is provided "as is"
	; and without warranties of any kind. In no event shall the owners or
	; contributors be liable for any damages of any kind, even if advised
	; of the possibility of such damage. See license for more information.

	; Please help us to improve this software by contributing changes and
	; fixes via http://www.rebol.com/feedback.html - Thanks!
]
;   The preprocessor includes the full power of REBOL, allowing you
;   to evaluate any REBOL expression during processing. The program
;   allows merging of code, data, images, sound and other types of
;   files into a single output file. It also supports conditional
;   expressions, such as the conditional including of files.
;   
;   The output is a reduced REBOL source file with extra spacing,
;   comments, and include file headers removed. Processing is
;   recursive, so included files can themselves contain commands to
;   process. This is very powerful.
;   
;   Note: REBOL/Encap includes the preprocessor built-in, so you
;   do not need to preprocess files as a separate step.
;
;   Usage:
;       From command line, shortcut, or alias:
;           rebol -s prerebol.r input.r output.r
;   
;       From REBOL console or script:
;           do/args %prerebol.r [%input.r %output.r]
;   
;       Supported preprocessor commands:
;           #include %file.r | (file-expr)
;           #include-files %path [%file1 %file2 %file3]
;           #include-string %file.txt | (file-expr)
;           #include-binary %file.bin | (file-expr)
;           #do [expr]
;           #if [expr] [then-block]
;           #either [expr] [then-block] [else-block]
;   
;       An unlimited number of expressions and results can be
;       processed and created by using the #do command. (So, if
;       you add new commands, please make sure they can't first
;       be done with #do. Each command affects use of #issue
;       data within source code, e.g. this preprocessor cannot
;       be preprocessed.)
;   }

print [
	system/script/header/title system/script/header/version newline
	"Copyright 2002 REBOL Technologies" newline
]

set [in-file out-file] system/script/args
if in-file = out-file [out-file: none]
if in-file [in-file: to-rebol-file in-file]
if out-file [out-file: to-rebol-file out-file]

all [ ; Deal with ROOT path problem when drag and drop arg:
	in-file
	what-dir = %/C/
	change-dir first split-path in-file
]

if not view? [
	request-file: func [/title quest opt /save] [
		opt: ask quest
		if empty? trim opt [return none]
		reduce [to-file opt]
	]
]

if none? in-file [
	in-file: request-file/title "Input file? " "Open"
	if none? in-file [quit]
	in-file: in-file/1
]

if none? out-file [
	out-file: request-file/title/save "Output file? " "Save"
	if none? out-file [quit]
	out-file: out-file/1
]

if in-file = out-file [
	if not confirm "Input and output files are the same. Are you sure?" [quit]
]

if not all [
	msg: "Missing input file name argument"
	in-file
	in-file: to-file in-file
	msg: reform ["Input file not found:" in-file]
	exists? in-file
	msg: "Missing output file name argument"
	out-file
	out-file: to-file out-file
	msg: "Input file is not a REBOL file"
	script? in-file
	msg: "Cannot load REBOL input file (may have errors)"
	not error? try [data: load/all in-file]
][
	print ["*** Preprocessor error:" msg]
	halt
]

; Allow it to run as script or as encap program:
test: #do [1]  ; Special check for encapped preprocessor
do either issue? test [%prebol.r][#include-string %prebol.r] ; avoid preproc

data: process-source/keep in-file

data: either find first :mold /flat [mold/only/flat data][mold/only data]
write out-file data

print [
	"Output to:" out-file newline
	"Processed:" size "bytes"
]

if system/version/4 = 3 [wait 2] ; wait on windows
